@@ -1,5 +1,5 @@
 /*
-pyjournalctl - Python module that reads systemd journald similar to journalctl
+pyjournalctl - Python module that reads systemd journal similar to journalctl
 Copyright (C) 2012  Steven Hiscocks
 
 This library is free software; you can redistribute it and/or
@@ -179,7 +179,7 @@ Journalctl___process_field(Journalctl *self, PyObject *key, const void *value, s
     if (PyDict_Check(self->call_dict))
         callable = PyDict_GetItem(self->call_dict, key);
 
-    if (callable) {
+    if (PyCallable_Check(callable)) {
 #if PY_MAJOR_VERSION >=3
         return_value = PyObject_CallFunction(callable, "y#", value, value_len);
 #else
@@ -188,7 +188,7 @@ Journalctl___process_field(Journalctl *self, PyObject *key, const void *value, s
         if (!return_value)
             PyErr_Clear();
     }
-    if (!return_value)
+    if (!return_value && PyCallable_Check(self->default_call))
 #if PY_MAJOR_VERSION >=3
         return_value = PyObject_CallFunction(self->default_call, "y#", value, value_len);
 #else
@@ -254,7 +254,6 @@ Journalctl_get_next(Journalctl *self, PyObject *args)
     const void *msg;
     size_t msg_len;
     const char *delim_ptr;
-    char bootid[33];
     PyObject *key, *value, *cur_value, *tmp_list;
 
     SD_JOURNAL_FOREACH_DATA(self->j, msg, msg_len) {
@@ -264,8 +263,6 @@ Journalctl_get_next(Journalctl *self, PyObject *args)
 #else
         key = PyString_FromStringAndSize(msg, delim_ptr - (const char*) msg);
 #endif
-        if (strncmp(msg, "_BOOT_ID", 8) == 0)
-            strncpy(bootid, (char *) delim_ptr + 1, 33);
         value = Journalctl___process_field(self, key, delim_ptr + 1, (const char*) msg + msg_len - (delim_ptr + 1) );
         if (PyDict_Contains(dict, key)) {
             cur_value = PyDict_GetItem(dict, key);
@@ -302,22 +299,20 @@ Journalctl_get_next(Journalctl *self, PyObject *args)
     }
 
     sd_id128_t sd_id;
-    if (sd_id128_from_string(bootid, &sd_id) == 0) {
-        uint64_t monotonic;
-        if (sd_journal_get_monotonic_usec(self->j, &monotonic, &sd_id) == 0) {
-            char monotonic_str[20];
-            sprintf(monotonic_str, "%llu", (long long unsigned) monotonic);
+    uint64_t monotonic;
+    if (sd_journal_get_monotonic_usec(self->j, &monotonic, &sd_id) == 0) {
+        char monotonic_str[20];
+        sprintf(monotonic_str, "%llu", (long long unsigned) monotonic);
 #if PY_MAJOR_VERSION >=3
-            key = PyUnicode_FromString("__MONOTONIC_TIMESTAMP");
+        key = PyUnicode_FromString("__MONOTONIC_TIMESTAMP");
 #else
-            key = PyString_FromString("__MONOTONIC_TIMESTAMP");
+        key = PyString_FromString("__MONOTONIC_TIMESTAMP");
 #endif
-            value = Journalctl___process_field(self, key, monotonic_str, strlen(monotonic_str));
+        value = Journalctl___process_field(self, key, monotonic_str, strlen(monotonic_str));
 
-            PyDict_SetItem(dict, key, value);
-            Py_DECREF(key);
-            Py_DECREF(value);
-        }
+        PyDict_SetItem(dict, key, value);
+        Py_DECREF(key);
+        Py_DECREF(value);
     }
 
     char *cursor;
@@ -434,6 +429,8 @@ Journalctl_add_matches(Journalctl *self, PyObject *args)
         arg = Py_BuildValue("OO", key, value);
         Journalctl_add_match(self, arg);
         Py_DECREF(arg);
+        if (PyErr_Occurred())
+            return NULL;
     }
 
     Py_DECREF(dict);
@@ -804,6 +801,8 @@ Journalctl_log_level(Journalctl *self, PyObject *args)
         arg = Py_BuildValue("(ss)", "PRIORITY", level_str);
         Journalctl_add_match(self, arg);
         Py_DECREF(arg);
+        if (PyErr_Occurred())
+            return NULL;
     }
     Py_RETURN_NONE;
 }
@@ -830,11 +829,10 @@ Journalctl_this_boot(Journalctl *self, PyObject *args)
 
     PyObject *arg;
     arg = Py_BuildValue("(ss)", "_BOOT_ID", bootid);
-    if (! Journalctl_add_match(self, arg)) {
-        Py_DECREF(arg);
-        return NULL;
-    }
+    Journalctl_add_match(self, arg);
     Py_DECREF(arg);
+    if (PyErr_Occurred())
+        return NULL;
 
     Py_RETURN_NONE;
 }
@@ -861,11 +859,10 @@ Journalctl_this_machine(Journalctl *self, PyObject *args)
 
     PyObject *arg;
     arg = Py_BuildValue("(ss)", "_MACHINE_ID", machineid);
-    if (! Journalctl_add_match(self, arg)) {
-        Py_DECREF(arg);
-        return NULL;
-    }
+    Journalctl_add_match(self, arg);
     Py_DECREF(arg);
+    if (PyErr_Occurred())
+        return NULL;
 
     Py_RETURN_NONE;
 }
@@ -920,7 +917,7 @@ Journalctl_set_call_dict(Journalctl *self, PyObject *value, void *closure)
     return 0;
 }
 
-static PyObject *
+/*static PyObject *
 Journalctl_get_data_threshold(Journalctl *self, void *closure)
 {
     size_t cvalue;
@@ -967,14 +964,14 @@ Journalctl_set_data_threshold(Journalctl *self, PyObject *value, void *closure)
         return -1;
     }
     return 0;
-}
+} */
 
 static PyGetSetDef Journalctl_getseters[] = {
-    {"data_threshold",
+/*    {"data_threshold",
     (getter)Journalctl_get_data_threshold,
     (setter)Journalctl_set_data_threshold,
     "data threshold",
-    NULL},
+    NULL},*/
     {"call_dict",
     (getter)Journalctl_get_call_dict,
     (setter)Journalctl_set_call_dict,
@@ -1069,7 +1066,7 @@ static PyTypeObject JournalctlType = {
 static PyModuleDef pyjournalctl_module = {
     PyModuleDef_HEAD_INIT,
     "pyjournalctl",
-    "Module that interfaces with journalctl.",
+    "Module that reads systemd journal similar to journalctl.",
     -1,
     NULL, NULL, NULL, NULL, NULL
 };
@@ -1099,14 +1096,14 @@ initpyjournalctl(void)
         return NULL;
 #else
     m = Py_InitModule3("pyjournalctl", NULL,
-                       "Module that interfaces with journalctl.");
+                   "Module that reads systemd journal similar to journalctl.");
     if (m == NULL)
         return;
 #endif
 
     Py_INCREF(&JournalctlType);
     PyModule_AddObject(m, "Journalctl", (PyObject *)&JournalctlType);
-    PyModule_AddStringConstant(m, "__version__", "0.6.1");
+    PyModule_AddStringConstant(m, "__version__", "0.7.0");
     PyModule_AddIntConstant(m, "SD_JOURNAL_NOP", SD_JOURNAL_NOP);
     PyModule_AddIntConstant(m, "SD_JOURNAL_APPEND", SD_JOURNAL_APPEND);
     PyModule_AddIntConstant(m, "SD_JOURNAL_INVALIDATE", SD_JOURNAL_INVALIDATE);
